#include "com_googlecode_mso_jinklevel_JInkLevel.h"
#include <inklevel.h>
#include <jni.h>
#include <stdlib.h>
#include <libintl.h>
#define _(String) dgettext ("jinklevel", String)

void throwByName
(
		JNIEnv* env,
		char* name,
		char* msg
)
{

	jclass cls = NULL;

	if (env == NULL) {
		printf("Want to throw a %s but JNIEnv is NULL\n", name);
		return;
	}

	if ((*env)->ExceptionOccurred(env)) {
		return;
	}

	cls = (*env)->FindClass(env, name);
	if (cls == NULL) {
		printf("FindClass failed for %s\n", name);
		return;
	}

	(*env)->ThrowNew(env, cls, msg);

	(*env)->DeleteLocalRef(env, cls);
}

JNIEXPORT jobject JNICALL Java_com_googlecode_mso_jinklevel_JInkLevel_listCartridges
(
		JNIEnv *env,
		jclass cls,
		jint portType,
		jstring deviceFile,
		jint portNumber
)
{

	jclass listCls = (*env)->FindClass(env, "java/util/ArrayList");
	if(listCls == NULL) {
		printf("FindClass failed for ArrayList\n");
		return NULL;
	}
	jmethodID listConst = (*env)->GetMethodID(env, listCls, "<init>", "()V");
	if(listConst == NULL) {
		printf("GetMethodID failed for ArrayList constructor\n");
		return NULL;
	}
	jobject list = (*env)->NewObject(env, listCls, listConst);
	if(list == NULL) {
		printf("Error constructing ArrayList\n");
		return NULL;
	}
	jmethodID addToList = (*env)->GetStaticMethodID(env, cls,"addToList", "(Ljava/util/List;Lcom/googlecode/mso/jinklevel/Cartridge;)V");
	if(addToList == NULL) {
		printf("GetMethodID failed for addToList\n");
		return NULL;
	}
	jclass cartridgeCls = (*env)->FindClass(env, "com/googlecode/mso/jinklevel/Cartridge");
	if(cartridgeCls == NULL) {
		printf("FindClass failed for Cartridge\n");
		return NULL;
	}
	jmethodID cartridgeConst = (*env)->GetMethodID(env, cartridgeCls,"<init>", "(II)V");
	if(cartridgeConst == NULL) {
		printf("GetMethodID failed for Cartridge constructor\n");
		return NULL;
	}

	char *c_deviceFile = (char *) (*env)->GetStringUTFChars(env, deviceFile,NULL);
	if(c_deviceFile == NULL)
		return NULL;
	char errorMessage[100];
	struct ink_level *level = NULL;

	level = (struct ink_level *) malloc(sizeof(struct ink_level));

	if (level == NULL) {
		throwByName(env, "java/lang/OutOfMemoryError", "Not enough memory available");
		return NULL;
	}

	int result = get_ink_level(portType, c_deviceFile, portNumber, level);

	if (result != OK) {
		switch (result) {
		case ERROR:
			sprintf(errorMessage, _("An unknown error occured"));
			break;
		case DEV_PARPORT_INACCESSIBLE:
		case DEV_LP_INACCESSIBLE:
		case DEV_USB_LP_INACCESSIBLE:
		case DEV_CUSTOM_USB_INACCESSIBLE:
			sprintf(errorMessage, _("Could not access device"));
			break;
		case COULD_NOT_GET_DEVICE_ID:
			sprintf(errorMessage, _("Could not get device id"));
			break;
		case UNKNOWN_PORT_SPECIFIED:
			sprintf(errorMessage, _("Unknown port specified"));
			break;
		case NO_PRINTER_FOUND:
			sprintf(errorMessage, _("No printer found"));
			break;
		case NO_DEVICE_CLASS_FOUND:
			sprintf(errorMessage, _("No device class found"));
			break;
		case NO_CMD_TAG_FOUND:
			sprintf(errorMessage, _("No cmd tag found"));
			break;
		case PRINTER_NOT_SUPPORTED:
			sprintf(errorMessage, _("Printer not supported"));
			break;
		case NO_INK_LEVEL_FOUND:
			sprintf(errorMessage, _("No ink level found"));
			break;
		case COULD_NOT_WRITE_TO_PRINTER:
			sprintf(errorMessage, _("Could not write to printer"));
			break;
		case COULD_NOT_READ_FROM_PRINTER:
			sprintf(errorMessage, _("Could not read from printer"));
			break;
		case COULD_NOT_PARSE_RESPONSE_FROM_PRINTER:
			sprintf(errorMessage, _("Could not parse response from printer"));
			break;
		case COULD_NOT_GET_CREDIT:
			sprintf(errorMessage, _("Could not get credit"));
			break;
		case BJNP_URI_INVALID:
			sprintf(errorMessage, _("Printer URI is invalid: '%s'"), c_deviceFile);
			break;
		case BJNP_INVALID_HOSTNAME:
			sprintf(errorMessage, _("Could not open hostname: '%s'"), c_deviceFile);
			break;
		}
		throwByName(env, "com/googlecode/mso/jinklevel/InkLevelException",errorMessage);
		free(level);
		return NULL;
	}

	if(level->status == RESPONSE_INVALID) {
		throwByName(env, "com/googlecode/mso/jinklevel/InkLevelException","No ink level found");
		free(level);
		return NULL;
	}


	int i;
	for(i = 0; i < MAX_CARTRIDGE_TYPES; i++) {
		if(level->levels[i][INDEX_TYPE] != CARTRIDGE_NOT_PRESENT) {
			jobject cartridge = (*env)->NewObject(env, cartridgeCls,cartridgeConst,level->levels[i][INDEX_TYPE],level->levels[i][INDEX_LEVEL]);
			if(cartridge == NULL) {
				printf("Error constructing Cartridge\n");
				return NULL;
			}
			(*env)->CallStaticVoidMethod(env, cls, addToList, list,cartridge);
			if ((*env)->ExceptionOccurred(env)) {
				printf("CallStaticVoidMethod failed for addToList");
				return NULL;
			}
		}
	}

	free(level);
	(*env)->ReleaseStringUTFChars(env, deviceFile, c_deviceFile);

	return list;
}

JNIEXPORT jint JNICALL JNI_OnLoad
(
		JavaVM *jvm,
		void *reserved
)
{
	bindtextdomain("jinklevel", "/usr/share/locale");
	return JNI_VERSION_1_4;
}
