package com.googlecode.mso.jinklevel;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * @author Serkan Kaba
 *
 */
public enum Color {
	/**
	 * Constant for black cartridge
	 */
	CARTRIDGE_BLACK,
	/**
	 * Constant for color cartridge
	 */
	CARTRIDGE_COLOR,
	/**
	 * Constant for photo cartridge
	 */
	CARTRIDGE_PHOTO,
	/**
	 * Constant for cyan cartridge
	 */
	CARTRIDGE_CYAN,
	/**
	 * Constant for magenta cartridge
	 */
	CARTRIDGE_MAGENTA,
	/**
	 * Constant for yellow cartridge
	 */
	CARTRIDGE_YELLOW,
	/**
	 * Constant for photoblack cartridge
	 */
	CARTRIDGE_PHOTOBLACK,
	/**
	 * Constant for photocyan cartridge
	 */
	CARTRIDGE_PHOTOCYAN,
	/**
	 * Constant for photomagenta cartridge
	 */
	CARTRIDGE_PHOTOMAGENTA,
	/**
	 * Constant for photoyellow cartridge
	 */
	CARTRIDGE_PHOTOYELLOW,
	/**
	 * Constant for red cartridge
	 */
	CARTRIDGE_RED,
	/**
	 * Constant for green cartridge
	 */
	CARTRIDGE_GREEN,
	/**
	 * Constant for blue cartridge
	 */
	CARTRIDGE_BLUE,
	/**
	 * Constant for light black cartridge
	 */
	CARTRIDGE_LIGHTBLACK,
	/**
	 * Constant for light cyan cartridge
	 */
	CARTRIDGE_LIGHTCYAN,
	/**
	 * Constant for light magenta cartridge
	 */
	CARTRIDGE_LIGHTMAGENTA,
	/**
	 * Constant for light light black cartridge
	 */
	CARTRIDGE_LIGHTLIGHTBLACK,
	/**
	 * Constant for matte black cartridge
	 */
	CARTRIDGE_MATTEBLACK,
	/**
	 * Constant for gloss optimizer cartridge
	 */
	CARTRIDGE_GLOSSOPTIMIZER,
	/**
	 * Constant unknown cartridge
	 */
	CARTRIDGE_UNKNOWN,
	/**
	 * Constant for 3-in-1 black, cyan, magenta cartridge
	 */
	CARTRIDGE_KCM,
	/**
	 * Constant for 3-in-1 black, 2x grey, black cartridge
	 */
	CARTRIDGE_GGK,
	/**
	 * Constant for 4-in-1 black, cyan, magenta, yellow cartridge
	 */
	CARTRIDGE_KCMY,
	/**
	 * Constant for 2-in-1 photocyan, photomagenta cartridge
	 */
	CARTRIDGE_LCLM,
	/**
	 * Constant for 2-in-1 yellow, magenta cartridge
	 */
	CARTRIDGE_YM,
	/**
	 * Constant for 2-in-1 cyan, black cartridge
	 */
	CARTRIDGE_CK,
	/**
	 * Constant for 2-in-1 light grey, photoblack cartridge
	 */
	CARTRIDGE_LGPK,
	/**
	 * Constant light grey cartridge
	 */
	CARTRIDGE_LG,
	/**
	 * Constant medium grey cartridge
	 */
	CARTRIDGE_G,
	/**
	 * Constant photogrey cartridge
	 */
	CARTRIDGE_PG,
	/**
	 * Constant white cartridge
	 */
	CARTRIDGE_WHITE;

	private static String[] colorNames={
		N_("Black"),
		N_("Color"),
		N_("Photo"),
		N_("Cyan"),
		N_("Magenta"),
		N_("Yellow"),
		N_("Photoblack"),
		N_("Photocyan"),
		N_("Photomagenta"),
		N_("Photoyellow"),
		N_("Red"),
		N_("Green"),
		N_("Blue"),
		N_("Light Black"),
		N_("Light Cyan"),
		N_("Light Magenta"),
		N_("Light Light Black"),
		N_("Matte Black"),
		N_("Gloss Optimizer"),
		N_("Unknown"),
		N_("Black, Cyan, Magenta"),
		N_("2x Grey and Black"),
		N_("Black, Cyan, Magenta, Yellow"),
		N_("Photocyan and Photomagenta"),
		N_("Yellow and Magenta"),
		N_("Cyan and Black"),
		N_("Light Grey and Photoblack"),
		N_("Light Grey"),
		N_("Medium Grey"),
		N_("Photogrey"),
		N_("White")
	};

	private static ResourceBundle res;
	
	private static String _(String s) {
		String translation = null;
		if(res == null)
			return s;
		try {
			translation = res.getString(s); 
		} catch(MissingResourceException e) {
		}
		if(translation == null)
			return s;
		else
			return translation;
	}
	private static String N_(String s) { return s; }

	@Override
	public String toString() {
		return _(colorNames[this.ordinal()]);
	}
	
	private static final String PATH = "com.googlecode.mso.jinklevel.l10n.Messages";

	/*
	 * The following code block is borrowed
	 * from Jubler project  http://www.jubler.org
	 */
	static {
		String lang = System.getProperty("user.language");
		String country = System.getProperty("user.country");

		setLang("_"+lang+"_"+country);
		if (res==null) {
			setLang("_"+lang);
			if (res==null) {
				setLang("");
			}
		}
	}

	private static void setLang(String langcode) {
		try {
			res = (ResourceBundle)Class.forName(PATH+langcode).newInstance();
		} catch (ClassNotFoundException e) {
		} catch (InstantiationException e) {
		} catch (IllegalAccessException e) {
		}
	}
}
